<?php

namespace Drupal\danse\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\danse\RecipientSelectionManager;
use Drupal\danse\Service;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Subscription Audit Notification Event settings for this site.
 */
final class Settings extends ConfigFormBase {

  /**
   * @var \Drupal\danse\Service
   */
  protected $danse;

  /**
   * @var \Drupal\danse\RecipientSelectionManager
   */
  protected $recipientSelectionManager;

  /**
   * Settings constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\danse\Service $danse
   * @param \Drupal\danse\RecipientSelectionManager $recipient_selection_manager
   */
  public function __construct(ConfigFactoryInterface $config_factory, Service $danse, RecipientSelectionManager $recipient_selection_manager) {
    parent::__construct($config_factory);
    $this->danse = $danse;
    $this->recipientSelectionManager = $recipient_selection_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('danse.service'),
      $container->get('danse.recipient.selection.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'danse_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['danse.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $form['recipient_selection_plugin'] = [
      '#type' => 'details',
      '#title' => $this->t('Recipient selection plugins'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];
    foreach ($this->danse->getPluginInstances() as $id => $plugin) {
      $form['recipient_selection_plugin'][$id] = [
        '#type' => 'select',
        '#title' => $plugin->label(),
        '#options' => $this->recipientSelectionManager->pluginListForSelect(),
        '#default_value' => $this->config('danse.settings')
          ->get('recipient_selection_plugin.' . $id),
        '#required' => TRUE,
      ];
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    /** @var \Drupal\Core\Config\Config $config */
    if ($config = $this->config('danse.settings')) {
      $config
        ->set('recipient_selection_plugin', $form_state->getValue('recipient_selection_plugin'))
        ->save();
    }
    parent::submitForm($form, $form_state);
  }

}
